// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
import { closeBrackets } from '@codemirror/autocomplete';
import { defaultKeymap } from '@codemirror/commands';
import { bracketMatching, foldGutter, indentOnInput, indentUnit } from '@codemirror/language';
import { Compartment, EditorState, StateEffect } from '@codemirror/state';
import { EditorView, highlightActiveLine, keymap, lineNumbers } from '@codemirror/view';
import { Mode } from './mode';
import { Theme } from './editortheme';
export var Configuration;
(function (Configuration) {
    /**
     * Builder for extensions from value of type T
     */
    class ExtensionBuilder {
        get extensionBuilder() {
            return this;
        }
    }
    /**
     * Extension builder that simply forwards a value as an extension.
     */
    class ExtensionForwarder extends ExtensionBuilder {
        constructor() {
            super();
        }
        of(value) {
            return value;
        }
    }
    /**
     * Extension builder that builds an extension from a facet.
     */
    class FacetWrapper extends ExtensionBuilder {
        constructor(facet) {
            super();
            this._facet = facet;
        }
        of(value) {
            return this._facet.of(value);
        }
    }
    /**
     * Extension builder that provides an extension depending
     * on a boolean value.
     */
    class ConditionalExtension extends ExtensionBuilder {
        constructor(truthy, falsy = []) {
            super();
            this._truthy = truthy;
            this._falsy = falsy;
        }
        of(value) {
            return value ? this._truthy : this._falsy;
        }
    }
    /**
     * Extension builds that provides an extension depending on
     * conditional function operating on a value.
     */
    class GenConditionalExtension extends ExtensionBuilder {
        constructor(fn, truthy, falsy = []) {
            super();
            this._fn = fn;
            this._builder = new ConditionalExtension(truthy, falsy);
        }
        of(value) {
            return this._builder.of(this._fn(value));
        }
    }
    /**
     * IConfigurableBuilder implementation *
     */
    class ConfigurableBuilder {
        constructor(builder) {
            this._compartment = new Compartment();
            this._builder = builder;
        }
        of(value) {
            return this._compartment.of(this._builder.of(value));
        }
        reconfigure(value) {
            return this._compartment.reconfigure(this._builder.of(value));
        }
    }
    /*
     * Specific builder for themes. Provide a default theme and
     * allows to register new themes.
     */
    class ThemeBuilder {
        constructor() {
            this._compartment = new Compartment();
        }
        of(value) {
            const v = value;
            return this._compartment.of(Theme.getTheme(v));
        }
        reconfigure(value) {
            const v = value;
            return this._compartment.reconfigure(Theme.getTheme(v));
        }
    }
    /**
     * Creates a ConfigurableBuilder based on an ExtensionForwarder.
     */
    function createForwarderBuilder() {
        return new ConfigurableBuilder(new ExtensionForwarder());
    }
    /**
     * Creates a ConfigurableBuilder based on a Facet.
     */
    function createConfigurableBuilder(facet) {
        return new ConfigurableBuilder(new FacetWrapper(facet));
    }
    /**
     * Creates a ConditionalBuilder from two extensions.
     */
    function createConditionalBuilder(truthy, falsy = []) {
        return new ConfigurableBuilder(new ConditionalExtension(truthy, falsy));
    }
    /**
     * Creates a ConditionalBuilder based on two extensions and a
     * conditional function.
     */
    function createGenConditionalBuilder(fn, truthy, falsy = []) {
        return new ConfigurableBuilder(new GenConditionalExtension(fn, truthy, falsy));
    }
    /**
     * Creates a theme builder.
     */
    function createThemeBuilder() {
        return new ThemeBuilder();
    }
    /**
     * Editor configuration: provides APIs to get and reconfigure CodeMirror 6
     * extensions from option names. Also allows to register new themes and
     * inject ne extensions in already configured editor instances.
     */
    class EditorConfiguration {
        constructor() {
            // Order does not matter
            this._configurableBuilderMap = new Map([
                ['tabSize', createConfigurableBuilder(EditorState.tabSize)],
                ['readOnly', createConfigurableBuilder(EditorState.readOnly)],
                ['keymap', createConfigurableBuilder(keymap)],
                ['indentUnit', createConfigurableBuilder(indentUnit)],
                ['smartIndent', createConditionalBuilder(indentOnInput())],
                ['autoClosingBrackets', createConditionalBuilder(closeBrackets())],
                ['matchBrackets', createConditionalBuilder(bracketMatching())],
                ['codeFolding', createConditionalBuilder(foldGutter())],
                ['styleActiveLine', createConditionalBuilder(highlightActiveLine())],
                ['lineNumbers', createConditionalBuilder(lineNumbers())],
                [
                    'lineWrap',
                    createGenConditionalBuilder((a) => a !== 'off', EditorView.lineWrapping)
                ],
                ['language', createForwarderBuilder()],
                ['theme', createThemeBuilder()]
            ]);
            this._themeOverloaderSpec = { '&': {}, '.cm-line': {} };
            this._themeOverloader = new Compartment();
        }
        /**
         * Reconfigures the extension mapped with key with the provided value.
         */
        reconfigureExtension(view, key, value) {
            const builder = this.get(key);
            if (builder) {
                view.dispatch({
                    effects: builder.reconfigure(value)
                });
            }
            else {
                const config = {};
                config[key] = value;
                const themeOverload = this.updateThemeOverload(config);
                view.dispatch({
                    effects: this._themeOverloader.reconfigure(themeOverload)
                });
            }
        }
        /**
         * Reconfigures all the extensions mapped with the options from the
         * provided partial configuration.
         */
        reconfigureExtensions(view, config) {
            const eff = [];
            for (const [key, value] of Object.entries(config)) {
                const builder = this.get(key);
                if (builder) {
                    eff.push(builder.reconfigure(value));
                }
            }
            const themeOverload = this.updateThemeOverload(config);
            eff.push(this._themeOverloader.reconfigure(themeOverload));
            view.dispatch({
                effects: eff
            });
        }
        /**
         * Appends extensions to the top-level configuration of the
         * editor.
         */
        injectExtension(view, ext) {
            view.dispatch({
                effects: StateEffect.appendConfig.of(ext)
            });
        }
        /**
         * Returns the list of initial extensions of an editor
         * based on the provided configuration.
         */
        getInitialExtensions(config) {
            const keys = Object.keys(config).filter(v => v !== 'insertSpaces' && v !== 'extraKeys');
            const extensions = [];
            for (const k of keys) {
                const builder = this.get(k);
                if (builder) {
                    const value = config[k];
                    extensions.push(builder.of(value));
                }
            }
            if (!config.theme) {
                extensions.push(Theme.defaultTheme());
            }
            const builder = this.get('keymap');
            const keymapExt = builder.of(config.extraKeys
                ? [...defaultKeymap, ...config.extraKeys]
                : [...defaultKeymap]);
            const indentBuilder = this.get('indentUnit');
            const insertExt = indentBuilder.of(config.insertSpaces ? ' '.repeat(config.tabSize) : '\t');
            let themeOverload = this.updateThemeOverload(config);
            extensions.push(this._themeOverloader.of(themeOverload), insertExt, keymapExt);
            Mode.ensure('text/x-python')
                .then(spec => {
                if (spec) {
                    extensions.push(this.get('language').of(spec.support));
                }
            })
                .catch(error => {
                console.error('Could not load language parser:');
                console.error(error);
            });
            return extensions;
        }
        updateThemeOverload(config) {
            const { fontFamily, fontSize, lineHeight, lineWrap, wordWrapColumn } = config;
            let needUpdate = false;
            const parentStyle = {};
            if (fontSize) {
                parentStyle.fontSize = fontSize + 'px';
                needUpdate = true;
            }
            if (fontFamily) {
                parentStyle.fontFamily = fontFamily;
                needUpdate = true;
            }
            if (lineHeight) {
                parentStyle.lineHeight = lineHeight.toString();
                needUpdate = true;
            }
            const lineStyle = {};
            if (lineWrap === 'wordWrapColumn') {
                lineStyle.width = wordWrapColumn + 'ch';
                needUpdate = true;
            }
            else if (lineWrap === 'bounded') {
                lineStyle.maxWidth = wordWrapColumn + 'ch';
                needUpdate = true;
            }
            if (needUpdate) {
                const newThemeSpec = {
                    '&': parentStyle,
                    '.cm-line': lineStyle
                };
                this._themeOverloaderSpec = {
                    ...this._themeOverloaderSpec,
                    ...newThemeSpec
                };
            }
            return EditorView.theme(this._themeOverloaderSpec);
        }
        get(key) {
            return this._configurableBuilderMap.get(key);
        }
    }
    Configuration.EditorConfiguration = EditorConfiguration;
})(Configuration || (Configuration = {}));
//# sourceMappingURL=editorconfiguration.js.map